<?php

namespace Drupal\datamaps\Element;

use Drupal\Component\Utility\Html;
use Drupal\Core\Render\Element\RenderElement;

/**
 * Renders a Datamap element.
 *
 * Display a datamap using the selected map variant, default is 'world'.
 *
 * For available options check https://github.com/markmarkoh/datamaps/tree/v0.5.9#default-options
 *
 * Usage example:
 * @code
 * $element['datamap'] = [
 *   '#type' => 'datamap',
 *   '#datamaps' => [
 *     'variant' => 'world',
 *     'options' => [...],
 *   ]
 * ];
 * @endcode
 *
 * @todo Add support for custom scope and dataUrl.
 * @todo Add support for variants.
 *
 * @RenderElement("datamap")
 */
class Datamap extends RenderElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#theme' => 'datamap',
      '#pre_render' => [
        [$class, 'preRender'],
      ],
      '#attributes' => [],
      '#datamaps' => [
        'variant' => 'world',
        'options' => [],
      ],
    ];
  }

  /**
   * Prepares the element for rendering.
   *
   * @param array $element
   *   The element render array.
   *
   * @return mixed
   */
  public static function preRender($element) {
    if (empty($element['#attributes']['id'])) {
      $element['#attributes']['id'] = Html::getUniqueId('datamaps' . time());
    }

    // Allow modules change the options for the datamap before passing it
    // to the Javascript layer.
    $selector = $element['#attributes']['id'];
    $hook_arguments = [$selector, $element['#datamaps']['options']];
    $options = \Drupal::moduleHandler()
      ->invokeAll('datamaps_options', $hook_arguments);
    $options['scope'] = 'world';

    $element['#attached'] = [
      'drupalSettings' => [
        'datamaps' => [
          [
            'selector' => $selector,
            'options' => $options,
          ],
        ],
      ],
      'library' => [
        'datamaps/' . $element['#datamaps']['variant'],
        'datamaps/script',
      ],
    ];

    return $element;
  }

}
